@@ -3,10 +3,11 @@
 # From https://github.com/aaugustin/websockets/blob/main/example/echo.py
 
 import asyncio
-import websockets
 import os
 
-LOCAL_WS_SERVER_PORT = int(os.environ.get('LOCAL_WS_SERVER_PORT', '8765'))
+import websockets
+
+LOCAL_WS_SERVER_PORT = int(os.environ.get("LOCAL_WS_SERVER_PORT", "8765"))
 
 
 async def echo(websocket, path):
@@ -18,4 +19,5 @@ async def main():
     async with websockets.serve(echo, "localhost", LOCAL_WS_SERVER_PORT):
         await asyncio.Future()  # run forever
 
+
 asyncio.run(main())
